"""Constants for the Canary integration."""

DOMAIN = "canary"

MANUFACTURER = "Canary Connect, Inc"

# Configuration
CONF_FFMPEG_ARGUMENTS = "ffmpeg_arguments"

# Data
DATA_COORDINATOR = "coordinator"
DATA_UNDO_UPDATE_LISTENER = "undo_update_listener"

# Defaults
DEFAULT_FFMPEG_ARGUMENTS = "-pred 1"
DEFAULT_TIMEOUT = 10
