"""Constants for Cloudflare."""

DOMAIN = "cloudflare"

# Config
CONF_RECORDS = "records"

# Data
DATA_UNDO_UPDATE_INTERVAL = "undo_update_interval"

# Defaults
DEFAULT_UPDATE_INTERVAL = 60  # in minutes

# Services
SERVICE_UPDATE_RECORDS = "update_records"
