"""Constants for Camera component."""
DOMAIN = "camera"

DATA_CAMERA_PREFS = "camera_prefs"

PREF_PRELOAD_STREAM = "preload_stream"

SERVICE_RECORD = "record"

CONF_LOOKBACK = "lookback"
CONF_DURATION = "duration"

CAMERA_STREAM_SOURCE_TIMEOUT = 10
CAMERA_IMAGE_TIMEOUT = 10
