"""Component for the Somfy MyLink device supporting the Synergy API."""

CONF_ENTITY_CONFIG = "entity_config"
CONF_SYSTEM_ID = "system_id"
CONF_REVERSE = "reverse"
CONF_DEFAULT_REVERSE = "default_reverse"
CONF_TARGET_NAME = "target_name"
CONF_REVERSED_TARGET_IDS = "reversed_target_ids"
CONF_TARGET_ID = "target_id"

DEFAULT_PORT = 44100

DATA_SOMFY_MYLINK = "somfy_mylink_data"
MYLINK_STATUS = "mylink_status"
DOMAIN = "somfy_mylink"

PLATFORMS = ["cover"]

MANUFACTURER = "Somfy"
