"""Constants for the Elgato Key Light integration."""

# Integration domain
DOMAIN = "elgato"

# Home Assistant data keys
DATA_ELGATO_CLIENT = "elgato_client"

# Attributes
ATTR_IDENTIFIERS = "identifiers"
ATTR_MANUFACTURER = "manufacturer"
ATTR_MODEL = "model"
ATTR_ON = "on"
ATTR_SOFTWARE_VERSION = "sw_version"

CONF_SERIAL_NUMBER = "serial_number"
