"""Constants for the Mill heater component."""

ATTR_AWAY_TEMP = "away_temp"
ATTR_COMFORT_TEMP = "comfort_temp"
ATTR_ROOM_NAME = "room_name"
ATTR_SLEEP_TEMP = "sleep_temp"
MANUFACTURER = "Mill"
MAX_TEMP = 35
MIN_TEMP = 5
DOMAIN = "mill"
SERVICE_SET_ROOM_TEMP = "set_room_temperature"
